-- Function that creates staging area temporary tables
-- This function should NOT be called directly. See load_dw().
CREATE OR REPLACE FUNCTION sa_create() returns void as $$
BEGIN
	PERFORM sa_inventory_create();
	PERFORM sa_net_usage_create();
END;
$$ language plpgsql;

CREATE OR REPLACE FUNCTION valid_macaddress(mac TEXT) RETURNS BOOLEAN AS $$
       SELECT $1 ~* '(([0-9]|[a-f]){2}:){5}([0-9]|[a-f]){2}' AND $1 <> '00:00:00:00:00:00';
$$ LANGUAGE SQL;

-- Creates inventory staging area. This function should NOT be called directly.
CREATE OR REPLACE FUNCTION sa_inventory_create() returns void as $$
DECLARE
	start_ts TIMESTAMP;
	total_rows INTEGER;
BEGIN

	-- getting initial timestamp
	start_ts = CLOCK_TIMESTAMP();
	
	-- preventing updates to occur
	LOCK TABLE telecenter_inventory IN EXCLUSIVE MODE;

	-- copying it to our brand new staging area table
	CREATE TABLE sa_inventory AS
		SELECT
			row_number() over() AS id, *
		FROM
			(SELECT
				DISTINCT
				t.id as tl_id,
				t.cit_id as cit_id,
				i.contact_date,
				i.machine_type,
				i.macaddr,
				i.os_type,
				i.os_distro,
				i.os_kernel,
				i.processor,
				i.memory,
				i.disk1_model,
				i.disk1_size,
				i.disk1_used,
				i.disk2_model,
				i.disk2_size,
				i.disk2_used,
				i.extra_hds,
				i.ts,
				'0'::boolean as valid
			FROM 
				telecenter_inventory i 
			LEFT JOIN
				dim_telecenter t
			ON
				i.m_superid = t.m_superid
			WHERE
				t.id IS NOT NULL
			) AS a;

	-- how many rows were inserted?
	GET DIAGNOSTICS total_rows = ROW_COUNT;

	-- creating primary key (and btree index)
	ALTER TABLE sa_inventory ADD PRIMARY KEY ("id");

	-- logging
	INSERT INTO control (name, start_time, end_time, total) 
		VALUES ('create sa inventory', start_ts, CLOCK_TIMESTAMP(), total_rows);

	-- truncating OLTP table
	TRUNCATE TABLE telecenter_inventory;

END;
$$ language plpgsql;


-- Creates net usage staging area. This function should NOT be called directly.
CREATE OR REPLACE FUNCTION sa_net_usage_create() returns void as $$
DECLARE
	start_ts TIMESTAMP;
	total_rows INTEGER;
BEGIN

	-- getting initial timestamp
	start_ts = CLOCK_TIMESTAMP();
	
	-- preventing updates to occur
	LOCK TABLE telecenter_net_usage IN EXCLUSIVE MODE;

	-- copying it to our brand new staging area table
	CREATE TABLE sa_net_usage AS
		SELECT
			row_number() over() AS id, *
		FROM
			(SELECT
				DISTINCT
				t.id as tl_id,
				n.contact_date,
				CAST(n.macaddr as MACADDR),
				n.collect_time,
				n.down_kbits,
				n.down_packages,
				n.up_kbits,
				n.up_packages,
				'1'::boolean as valid
			FROM 
				telecenter_net_usage n
			LEFT JOIN 
				dim_telecenter t 
			ON
				substring(n.tl_hash from 1 for 33) = t.hash AND substring(n.tl_hash from 34 for 5) = t.superid
			WHERE
				n.down_kbits <> 0 AND n.down_packages <> 0 AND up_kbits <> 0 AND up_packages <> 0 AND t.id IS NOT NULL
			) AS a;

	-- how many rows were inserted?
	GET DIAGNOSTICS total_rows = ROW_COUNT;

	-- creating the primary key
	--ALTER TABLE sa_net_usage ADD PRIMARY KEY ("id");

	-- logging
	INSERT INTO control (name, start_time, end_time, total) 
		VALUES ('create sa net usage', start_ts, CLOCK_TIMESTAMP(), total_rows);


	-- truncating OLTP table
	TRUNCATE TABLE telecenter_net_usage;

END;
$$ language plpgsql;


-- sanitize, removing invalid rows
CREATE OR REPLACE FUNCTION sa_sanitize() returns void as $$
BEGIN

	PERFORM sa_sanitize_inventory();
	PERFORM sa_sanitize_net_usage();

END;
$$ language plpgsql;

/****************************** Ver que rumo terá os rejected *********************************/

-- sanitizes sa_inventory table data
CREATE OR REPLACE FUNCTION sa_sanitize_inventory() returns void as $$
DECLARE
	start_ts TIMESTAMP;
	total_rows INTEGER;
	new_inv record;
BEGIN
	-- getting initial timestamp
	start_ts = CLOCK_TIMESTAMP();


	-- we create this table marking as rows as invalid.
	-- when a computer sends more than one inventory per day, we get
	-- only the last one. Validating the newers:
	FOR new_inv IN
		SELECT max(id) AS id FROM sa_inventory GROUP BY tl_id, macaddr, contact_date
	LOOP
		UPDATE sa_inventory set valid = '1' WHERE id = new_inv.id;
	END LOOP;

	-- Checking whether valid records have the required information
	UPDATE 	sa_inventory
	SET	valid = '0'
	WHERE
		valid = '1' AND
		(tl_id is NULL OR
		contact_date is NULL OR
		os_type is NULL OR 
		os_distro is NULL OR 
		os_kernel is NULL OR 
		processor is NULL OR 
		memory is NULL OR
		disk1_model is NULL OR 
		disk1_size is NULL OR 
		disk1_used is NULL OR
		NOT valid_macaddress(macaddr) OR
		machine_type < 0 OR machine_type > 1);


	-- log the invalid information into rejected table
	INSERT INTO rejected_inventory 
		(tl_id, contact_date, machine_type, macaddr, 
		os_type, os_distro, os_kernel, processor, memory, disk1_model, 
		disk1_size, disk1_used, disk2_model, disk2_size, disk2_used)
	(SELECT
		tl_id, contact_date, machine_type, macaddr, 
		os_type, os_distro, os_kernel, processor, memory, disk1_model, 
		disk1_size, disk1_used, disk2_model, disk2_size, disk2_used
	FROM
		sa_inventory
	WHERE
		valid = '0');
		
	-- how many rows we inserted?
	GET DIAGNOSTICS total_rows = ROW_COUNT;

	-- logging
	INSERT INTO control (name, start_time, end_time, total) 
		VALUES ('sanitize sa inventory', start_ts, CLOCK_TIMESTAMP(), total_rows);

END;
$$ language plpgsql;


-- sanitize the data in the net_usage staging area. Invalid data
-- will be marked using the column 'valid' and inserted into the
-- rejected_net_usage table.
CREATE OR REPLACE FUNCTION sa_sanitize_net_usage() returns void as $$
DECLARE
	start_ts TIMESTAMP;
	total_rows INTEGER;
BEGIN
	-- getting initial timestamp
	start_ts = CLOCK_TIMESTAMP();
	
	-- check required information updating the valid column
	UPDATE	sa_net_usage
	SET 	valid = '0'
	WHERE 
		tl_id is NULL OR
		contact_date is NULL OR
		collect_time is NULL OR
		NOT valid_macaddress(macaddr::text);

	-- log the invalid information into rejected table
	INSERT INTO rejected_net_usage 
		(tl_id, contact_date, macaddr, collect_time, 
		down_kbits, down_packages, up_kbits, up_packages)
	(SELECT
		tl_id, contact_date, macaddr, collect_time, 
		down_kbits, down_packages, up_kbits, up_packages
	FROM
		sa_net_usage
	WHERE
		valid = '0');
		
	-- how many rows we inserted?
	GET DIAGNOSTICS total_rows = ROW_COUNT;

	-- logging
	INSERT INTO control (name, start_time, end_time, total) 
		VALUES ('sanitize sa net_usage', start_ts, CLOCK_TIMESTAMP(), total_rows);
END;
$$ language plpgsql;


-- Function that drops staging area tables.
-- This function should NOT be called directly.
CREATE OR REPLACE FUNCTION sa_drop() returns void as $$
DECLARE
	start_ts TIMESTAMP;
	total_rows INTEGER;
BEGIN
	-- getting initial timestamp
	start_ts = CLOCK_TIMESTAMP();
	
	DROP TABLE sa_inventory;

	-- how many rows we deleted?
	GET DIAGNOSTICS total_rows = ROW_COUNT;

	-- logging
	INSERT INTO control (name, start_time, end_time, total) 
		VALUES ('drop sa inventory', start_ts, CLOCK_TIMESTAMP(), total_rows);

	-- getting initial timestamp
	start_ts = CLOCK_TIMESTAMP();
	
	DROP TABLE sa_net_usage;

	-- how many rows we deleted?
	GET DIAGNOSTICS total_rows = ROW_COUNT;

	-- logging
	INSERT INTO control (name, start_time, end_time, total) 
		VALUES ('drop sa usage', start_ts, CLOCK_TIMESTAMP(), total_rows);
END;
$$ language plpgsql;
